;;; DO NOT MODIFY THIS FILE
(if (featurep 'x-symbol-autoloads) (error "Already loaded"))

;;;### (autoloads nil "_pkg" "lisp/_pkg.el")

(package-provide 'x-symbol :version 1.03 :type 'regular)

;;;***

;;;### (autoloads (x-symbol-initialize x-symbol-register-language x-symbol-fontify turn-on-x-symbol-conditionally x-symbol-mode x-symbol-key-autoload x-symbol-auto-mode-suffixes) "x-symbol-hooks" "lisp/x-symbol-hooks.el")

(autoload 'x-symbol-auto-mode-suffixes "x-symbol-hooks" "\
Return REGEXPs of three-value elements in `auto-mode-alist'.
These REGEXPs are added to SUFFIXES." nil nil)

(autoload 'x-symbol-key-autoload "x-symbol-hooks" "\
Initialize package x-symbol and use the keys for this command again.
Package x-symbol and the functions in `x-symbol-load-hook' should
re-bind all key-sequence which invoke this command.  You should provide
a prefix argument ARG to this command if `x-symbol-auto-key-autoload' is
nil." t nil)

(defalias 'x-symbol-map-autoload 'x-symbol-key-autoload)

(autoload 'x-symbol-mode "x-symbol-hooks" "\
Toggle X-Symbol mode.
Toggle X-Symbol mode.  If provided with a prefix argument, turn X-Symbol
mode on if the numeric value of the argument is positive, else turn it
off.  If no token language can be deduced, ask for a token language; if
provided with a non-numeric prefix argument, always ask.

By default, X-Symbol mode is disabled in special major-modes visiting a
file, e.g., `vm-mode'.  Use a prefix argument to be asked whether to
turn in on anyway.

When not already defined, various buffer-local variables are set when
turning on X-Symbol.  See `x-symbol-auto-style-alist' and the language
access `x-symbol-LANG-modes'.

Turning X-Symbol mode on requires a valid `x-symbol-language' and also
decodes tokens if the mode was turned off before, see
\\[x-symbol-decode-recode].  Turning X-Symbol mode off also encodes
x-symbol characters if the mode was turned on before, see
\\[x-symbol-encode-recode].  If optional argument SPECIAL has value
`init', the old mode status is assumed to be off." t nil)

(autoload 'turn-on-x-symbol-conditionally "x-symbol-hooks" "\
Turn on x-symbol mode conditionally, see `x-symbol-mode'.
Call `x-symbol-mode' with SPECIAL having value `init'." nil nil)

(autoload 'x-symbol-fontify "x-symbol-hooks" "\
Re-fontify region between BEG and END." t nil)

(autoload 'x-symbol-register-language "x-symbol-hooks" "\
Register token language LANGUAGE.
FEATURE is a feature which `provide's LANGUAGE.  MODES are major modes
which typically use LANGUAGE.  Using LANGUAGE's accesses will initialize
LANGUAGE, see `x-symbol-language-value'." nil nil)

(autoload 'x-symbol-initialize "x-symbol-hooks" "\
Initialize package X-Symbol.
See variable `x-symbol-initialize' and function `x-symbol-after-init'.
Also allocate colormap, see `x-symbol-image-colormap-allocation'.
Unless optional argument ARG is non-nil, do not initialize package
X-Symbol twice." t nil)

;;;***

;;;### (autoloads (x-symbol-image-editor x-symbol-image-delete-extents x-symbol-image-after-change-function x-symbol-image-parse-buffer) "x-symbol-image" "lisp/x-symbol-image.el")

(autoload 'x-symbol-image-parse-buffer "x-symbol-image" "\
*Parse buffer to find image insertion commands.
Parse buffer to display glyphs at the end of image insertion commands.
Image files are converted to \"image cache files\" with images not
bigger than `x-symbol-image-max-width' and `x-symbol-image-max-height'
having a image format XEmacs understands.  The conversion is done by a
program determined by `x-symbol-image-converter', currently you need
\"convert\" from ImageMagick.  To make this conversion fast, we use
asynchronous processes and two cache hierarchies:

 * Memory cache (`x-symbol-image-memory-cache'): buffer-local alist
   FILE.eps -> GLYPH, see also `x-symbol-image-use-remote'.
 * File cache: the image cache file, mentioned above, are kept, see also
   `x-symbol-image-update-cache', which is shadowed by a non-nil
   UPDATE-CACHE and `x-symbol-image-cache-directories'.

When the mouse is over an image insertion command, it is highlighted.
button2 starts an image editor, see `x-symbol-image-editor-alist'.
button3 pops up a menu, see `x-symbol-image-menu'.

The image insertion commands are recognized by keywords in the language
access `x-symbol-LANG-image-keywords' whose value have the form
  (IMAGE-REGEXP KEYWORD ...)
IMAGE-REGEXP should match all images files and is used to initialize the
buffer local memory cache, see `x-symbol-image-init-memory-cache'.

Each KEYWORD looks like (REGEXP [FUNCTION] ARG...).  Image insertion
commands matched by REGEXP are highlighted.  FUNCTION, which defaults to
`x-symbol-image-default-file-name', is called with ARGs to get the file
name of the corresponding image file.  If FUNCTION returns nil, the
command is not highlighted.

Relative image file names are expanded in the directory returned by the
function in the language access `x-symbol-LANG-master-directory', value
nil means function `default-directory'.  Implicitly relative image file
names are searched in a search path, see `x-symbol-image-use-remote'." t nil)

(autoload 'x-symbol-image-after-change-function "x-symbol-image" "\
Function in `after-change-functions' for image insertion commands." nil nil)

(autoload 'x-symbol-image-delete-extents "x-symbol-image" "\
Delete x-symbol image extents covering text between BEG and END.
See also `x-symbol-image-buffer-extents'." nil nil)

(autoload 'x-symbol-image-editor "x-symbol-image" "\
Start image editor for the image file FILE used in BUFFER.
If BUFFER is nil, just return string describing the command.  See
`x-symbol-image-editor-alist' and `x-symbol-image-current-marker'." t nil)

;;;***

;;;### (autoloads (x-symbol-tex-auto-coding-alist) "x-symbol-tex" "lisp/x-symbol-tex.el")

(autoload 'x-symbol-tex-auto-coding-alist "x-symbol-tex" "\
Find encoding in file `x-symbol-tex-coding-master'.
For ALIST and LIMIT, see `x-symbol-auto-coding-alist'." nil nil)

;;;***

;;;### (autoloads (x-symbol-variable-interactive) "x-symbol-vars" "lisp/x-symbol-vars.el")

(autoload 'x-symbol-variable-interactive "x-symbol-vars" "\
Provide interactive specification for `set-variable'.
VAR's options has been defined with `x-symbol-define-user-options'." nil nil)

;;;***

(provide 'x-symbol-autoloads)
